@@ -20,123 +20,14 @@
 */
 
 /*
- *  $Id: dataparser.cpp,v 1.16 2004/01/25 11:07:13 cepek Exp $
+ *  $Id: dataparser.cpp,v 1.20 2004/04/21 16:49:39 cepek Exp $
  */
 
-// #########################################################################
-#ifdef GNU_Gama_DataParser_demo
-
-#include <gnu_gama/xml/dataparser.h>
-#include <gnu_gama/list.h>
-#include <gamalib/language.h>
-#include <cstring>
-#include <cctype>
-#include <iostream>
-
-
-int main()
-{
-  using namespace std;
-  using namespace GNU_gama;
-  
-  const char* xml_input_data = 
-
-    "<?xml version=\"1.0\" ?>\n"
-    "<!DOCTYPE gnu-gama-data SYSTEM \"gnu-gama-data.dtd\">\n\n"
-
-    "<gnu-gama-data>\n\n"
-    "<text>\n"
-    "qwerty ...\n"
-    "asdfgh ...\n"
-    "zxcvbn ...\n"
-    "</text>\n\n"
-
-    "<adj-input-data>\n"
-    "  <sparse-mat>\n"
-    "    <rows>3</rows> <cols>2</cols> <nonz>999</nonz>\n"
-    "      <row> <nonz>1</nonz> <int>1</int> <flt>1.1</flt></row>\n"
-    "      <row> <nonz>1</nonz> <int>2</int> <flt>2.2</flt></row>\n"
-    "      <row> <nonz>2</nonz> <int>1</int> <flt>3.3</flt>      \n"
-    "                           <int>2</int> <flt>4.4</flt></row>\n"
-    "  </sparse-mat>\n\n"
-
-    "  <block-diagonal>\n"
-    "    <blocks>1</blocks> <nonz>678</nonz>\n"
-    "      <block> <dim>3</dim> <width>0</width>\n"
-    "      <flt>1.01</flt>"
-    "      <flt>2.01</flt>"
-    "      <flt>3.01</flt>"
-    "      </block>\n"
-    "  </block-diagonal>\n\n"
-
-    "     <vector>\n"
-    "       <dim>3</dim>\n"
-    "         <flt>10.1</flt>\n"
-    "         <flt>10.2</flt>\n"
-    "         <flt>10.3</flt>\n"
-    "     </vector>\n\n"
-
-    "     <array>\n"
-    "       <dim>2</dim>\n"
-    "         <int>1</int>\n"
-    "         <int>2</int>\n"
-    "     </array>\n"
-
-    "</adj-input-data>\n"
-
-    "<text>\n"
-    "This is a DataParser demo ...\n"
-    "</text>\n\n"
-    
-    "\n</gnu-gama-data>\n\n"
-    ;
-
-  try 
-    {
-      GaMaLib::set_gama_language(GaMaLib::en);
-
-      List<DataObject::Base*> objects;
-      DataParser dp(objects);
-      dp.xml_parse(xml_input_data, strlen(xml_input_data), 1);
-      
-      cout << DataObject::Base::xml_begin();
-
-      for (List<DataObject::Base*>::const_iterator i=objects.begin(); 
-           i!=objects.end(); ++i)
-        {
-          cout << (*i)->xml();
-          delete *i;
-        }
-
-      cout << DataObject::Base::xml_end();
-    }
-  catch(Exception::parser e)
-    {
-      cout << "\nOn line " << e.line 
-           << " --- exception : <"   << e.str 
-           << ">  --- error code : " << e.error_code << endl;
-      return 1;
-    }
-  catch(Exception::string g)
-    {
-      cout << "\nGaMaLib Exception : " << g.str << endl;
-      return 2;
-    }
-  catch(...)
-    {
-      cout << "\nUnknown exception\n";
-      return 3;
-    }
-
-  return 0;
-}
-
-#else
-// #########################################################################
 
 
 #include <gnu_gama/xml/dataparser.h>
-#include <gnu_gama/g3/g3_observation/g3_cluster_vec.h>
+#include <gnu_gama/gon2deg.h>
+#include <gnu_gama/radian.h>
 #include <cstring>
 
 using namespace std;
@@ -165,7 +56,6 @@ DataParser::~DataParser()
 DataParser::DataParser(List<DataObject::Base*>& obs) : objects(obs)
 {
   g3model = 0;
-  g3variance = 0;
 
   adj_sparse_mat = 0;
   adj_block_diagonal = 0;
@@ -173,6 +63,9 @@ DataParser::DataParser(List<DataObject::Base*>& obs) : objects(obs)
 
   point = 0;
 
+  optional(from_dh);
+  optional(to_dh);
+
   // initial parser state and implicit handlers
   
   state = s_start;
@@ -192,350 +85,336 @@ DataParser::DataParser(List<DataObject::Base*>& obs) : objects(obs)
 
   // .....  <gnu-gama-data>  .........................................
 
-  init(  s_start, t_gama_data, 
-       //---------------------
+  init(s_start, t_gama_data, 
        s_gama_data, 0, s_stop,
        &DataParser::gama_data, 0, 0);
 
 
   // .....  <g3-model>  ..............................................
 
-  init(  s_gama_data, t_g3_model,
-         //----------------------
-         s_g3_model, 0, 0,
-         &DataParser::g3_model, 0, &DataParser::g3_model);
+  init(s_gama_data, t_g3_model,
+       s_g3_model, 0, 0,
+       &DataParser::g3_model, 0, &DataParser::g3_model);
+
+  // .....  <g3-model> <unused | fixed | free | constr>  .............
+
+  init(s_g3_model, t_unused,
+       s_g3_param, 0, s_g3_model,
+       &DataParser::g3_param_unused, 0, 0);
+
+  init(s_g3_model, t_fixed,
+       s_g3_param, 0, s_g3_model,
+       &DataParser::g3_param_fixed, 0, 0);
+
+  init(s_g3_model, t_free,
+       s_g3_param, 0, s_g3_model,
+       &DataParser::g3_param_free, 0, 0);
+
+  init(s_g3_model, t_constr,
+       s_g3_param, 0, s_g3_model,
+       &DataParser::g3_param_constr, 0, 0);
+
+  init(s_g3_param, t_n,
+       s_g3_param_n, 0, 0,
+       0, 0, &DataParser::g3_param_n);
+
+  init(s_g3_param, t_e,
+       s_g3_param_e, 0, 0,
+       0, 0, &DataParser::g3_param_e);
+
+  init(s_g3_param, t_u,
+       s_g3_param_u, 0, 0,
+       0, 0, &DataParser::g3_param_u);
 
   // .....  <g3-model> <point>  .....................................
 
-  init(  s_g3_model, t_point,
-         //------------------
-         s_g3_point_1, s_g3_point_2, s_g3_model,
-         0, 0, 0);
+  init(s_g3_model, t_point,
+       s_g3_point_1, s_g3_point_2, s_g3_model,
+       0, 0, &DataParser::g3_point);
 
-  init(  s_g3_point_1, t_id,
-         //-----------------
-         s_g3_point_id, 0, s_g3_point_2,
-         0, &DataParser::add_text, &DataParser::g3_point_id);
+  init(s_g3_point_1, t_id,
+       s_g3_point_id, 0, s_g3_point_2,
+       0, &DataParser::add_text, &DataParser::g3_point_id);
 
-  init(  s_g3_point_2, t_x,
-         //----------------
-         s_g3_point_x, 0, s_g3_point_after_x,
-         0, &DataParser::add_text, &DataParser::append_sp);
 
-  init(  s_g3_point_after_x, t_y,
-         //----------------------
-         s_g3_point_y, 0, s_g3_point_after_y,
-         0, &DataParser::add_text, &DataParser::append_sp);
 
-  init(  s_g3_point_after_y, t_z,
-         //----------------------
-         s_g3_point_z, 0, s_g3_point_2,
-         0, &DataParser::add_text, &DataParser::g3_point_z);
+  init(s_g3_point_2, t_b,
+       s_g3_point_b, 0, s_g3_point_after_b,
+       0, &DataParser::add_text, &DataParser::g3_point_b);
 
-  init(  s_g3_point_2, t_height,
-         //---------------------
-         s_g3_point_height, 0, s_g3_point_2,
-         0, &DataParser::add_text, &DataParser::g3_point_height);
+  init(s_g3_point_after_b, t_l,
+       s_g3_point_l, 0, s_g3_point_after_l,
+       0, &DataParser::add_text, &DataParser::g3_point_l);
 
-  init(  s_g3_point_2, t_unused,
-         //---------------------
-         s_g3_point_unused, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_unused);
+  init(s_g3_point_after_l, t_h,
+       s_g3_point_h, 0, s_g3_point_2,
+       0, &DataParser::add_text, &DataParser::g3_point_h);
 
-  init(  s_g3_point_2, t_fixed,
-         //--------------------
-         s_g3_point_fixed, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_fixed);
+  // ++++++++++++++++++++++++++++++++
+  init(s_g3_point_2, t_x,
+       s_g3_point_x, 0, s_g3_point_after_x,
+       0, &DataParser::add_text, 0);
 
-  init(  s_g3_point_2, t_fixed_p,
-         //----------------------
-         s_g3_point_fixed_p, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_fixed_p);
+  init(s_g3_point_after_x, t_y,
+       s_g3_point_y, 0, s_g3_point_after_y,
+       0, &DataParser::add_text, 0);
 
-  init(  s_g3_point_2, t_fixed_h,
-         //----------------------
-         s_g3_point_fixed_h, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_fixed_h);
+  init(s_g3_point_after_y, t_z,
+       s_g3_point_z, 0, s_g3_point_2,
+       0, &DataParser::add_text, &DataParser::g3_point_z);
 
-  init(  s_g3_point_2, t_free,
-         //--------------------
-         s_g3_point_free, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_free);
+  // --------------------------------
+  init(s_g3_point_2, t_height,
+       s_g3_point_height, 0, s_g3_point_2,
+       0, &DataParser::add_text, &DataParser::g3_point_height);
 
-  init(  s_g3_point_2, t_free_p,
-         //----------------------
-         s_g3_point_free_p, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_free_p);
+  init(s_g3_point_2, t_unused,
+       s_g3_point_param, 0, s_g3_point_2,
+       &DataParser::g3_param_unused, 0, 0);
 
-  init(  s_g3_point_2, t_free_h,
-         //----------------------
-         s_g3_point_free_h, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_free_h);
+  init(s_g3_point_2, t_fixed,
+       s_g3_point_param, 0, s_g3_point_2,
+       &DataParser::g3_param_fixed, 0, 0);
 
-  init(  s_g3_point_2, t_constr,
-         //--------------------
-         s_g3_point_constr, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_constr);
+  init(s_g3_point_2, t_free,
+       s_g3_point_param, 0, s_g3_point_2,
+       &DataParser::g3_param_free, 0, 0);
 
-  init(  s_g3_point_2, t_constr_p,
-         //----------------------
-         s_g3_point_constr_p, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_constr_p);
+  init(s_g3_point_2, t_constr,
+       s_g3_point_param, 0, s_g3_point_2,
+       &DataParser::g3_param_constr, 0, 0);
 
-  init(  s_g3_point_2, t_constr_h,
-         //----------------------
-         s_g3_point_constr_h, 0, s_g3_point_2,
-         0, 0, &DataParser::g3_point_constr_h);
+  init(s_g3_point_param, t_n,
+       s_g3_point_param_n, 0, 0,
+       0, 0, &DataParser::g3_point_param_n);
 
-  // .....  <g3-model> <vector>  ....................................
+  init(s_g3_point_param, t_e,
+       s_g3_point_param_e, 0, 0,
+       0, 0, &DataParser::g3_point_param_e);
 
-  init(  s_g3_model, t_vector,
-         //-------------------
-         s_g3_vector, s_g3_vector_after_czz, 0,
-         &DataParser::g3_vector, 0, &DataParser::g3_vector);
-
-  init(  s_g3_vector, t_from,
-         //------------------
-         s_g3_vector_from, 0, s_g3_vector_after_from,
-         0, &DataParser::add_text, &DataParser::g3_from);
-
-  init(  s_g3_vector_after_from, t_to,
-         //---------------------------
-         s_g3_vector_to, 0, s_g3_vector_after_to,
-         0, &DataParser::add_text, &DataParser::g3_to);
-
-  init(  s_g3_vector_after_from, t_to,
-         //---------------------------
-         s_g3_vector_to, 0, s_g3_vector_after_to,
-         0, &DataParser::add_text, &DataParser::g3_to);
-
-  init(  s_g3_vector_after_to, t_dx,
-         //-------------------------
-         s_g3_vector_dx, 0, s_g3_vector_after_dx,
-         0, &DataParser::add_text, &DataParser::append_sp);
-
-  init(  s_g3_vector_after_dx, t_dy,
-         //-------------------------
-         s_g3_vector_dy, 0, s_g3_vector_after_dy,
-         0, &DataParser::add_text, &DataParser::append_sp);
-
-  init(  s_g3_vector_after_dy, t_dz,
-         //-------------------------
-         s_g3_vector_dz, 0, s_g3_vector_after_dz,
-         0, &DataParser::add_text, &DataParser::append_sp);
-
-  init(  s_g3_vector_after_dz, t_cxx,
-         //--------------------------
-         s_g3_vector_cxx, 0, s_g3_vector_after_cxx,
-         0, &DataParser::add_text, &DataParser::append_sp);
-
-  init(  s_g3_vector_after_cxx, t_cxy,
-         //---------------------------
-         s_g3_vector_cxy, 0, s_g3_vector_after_cxy,
-         0, &DataParser::add_text, &DataParser::append_sp);
-
-  init(  s_g3_vector_after_cxy, t_cxz,
-         //---------------------------
-         s_g3_vector_cxz, 0, s_g3_vector_after_cxz,
-         0, &DataParser::add_text, &DataParser::append_sp);
-
-  init(  s_g3_vector_after_cxz, t_cyy,
-         //---------------------------
-         s_g3_vector_cyy, 0, s_g3_vector_after_cyy,
-         0, &DataParser::add_text, &DataParser::append_sp);
-
-  init(  s_g3_vector_after_cyy, t_cyz,
-         //---------------------------
-         s_g3_vector_cyz, 0, s_g3_vector_after_cyz,
-         0, &DataParser::add_text, &DataParser::append_sp);
-
-  init(  s_g3_vector_after_cyz, t_czz,
-         //---------------------------
-         s_g3_vector_czz, 0, s_g3_vector_after_czz,
-         0, &DataParser::add_text, &DataParser::append_sp);
+  init(s_g3_point_param, t_u,
+       s_g3_point_param_u, 0, 0,
+       0, 0, &DataParser::g3_point_param_u);
 
   // .....  <g3-model> <obs>  ........................................
-
-  init(  s_g3_model, t_obs,
-         //------------------
-         s_g3_obs, 0, 0,
-         &DataParser::g3_obs, 0, &DataParser::g3_obs,
-         s_g3_obs_after_covmat);
+  
+  init(s_g3_model, t_obs,
+       s_g3_obs, 0, 0,
+       &DataParser::g3_obs, 0, &DataParser::g3_obs);
          
   // .....  <g3-model> <obs> <cov-mat>  ..............................
 
   init(s_g3_obs, t_covmat,
-       s_g3_obs_covmat, s_g3_obs_covmat_after_band, s_g3_obs_after_covmat,
+       s_g3_obs_covmat, s_g3_obs_covmat_after_band, 0,
        0, 0, &DataParser::g3_obs_cov);
 
   init(s_g3_obs_covmat, t_dim,
        s_g3_obs_covmat_dim, 0, s_g3_obs_covmat_after_dim,
-       0, &DataParser::add_text, &DataParser::append_sp);
+       0, &DataParser::add_text, 0);
 
   init(s_g3_obs_covmat_after_dim, t_band,
        s_g3_obs_covmat_band, 0, s_g3_obs_covmat_after_band,
-       0, &DataParser::add_text, &DataParser::append_sp);
+       0, &DataParser::add_text, 0);
 
   init(s_g3_obs_covmat_after_band, t_flt,
        s_g3_obs_covmat_flt, 0, s_g3_obs_covmat_after_band,
-       0, &DataParser::add_text, &DataParser::append_sp);
+       0, &DataParser::add_text, 0);
 
   // .....  <g3-model> <obs> <distance>  .............................
 
   init(s_g3_obs, t_dist,
-       s_g3_obs_dist, s_g3_obs_dist_has_val, 0,
-       0, 0, &DataParser::g3_obs_dist,
-       s_g3_obs_dist_has_variance);
+       s_g3_obs_dist, s_g3_obs_dist_opt, 0,
+       0, 0, &DataParser::g3_obs_dist);
 
   init(s_g3_obs_dist, t_from,
        s_g3_obs_dist_from, 0, s_g3_obs_dist_after_from,
-       0, &DataParser::add_text, &DataParser::g3_from);
+       0, &DataParser::add_text, 0);
 
   init(s_g3_obs_dist_after_from, t_to,
        s_g3_obs_dist_to, 0, s_g3_obs_dist_after_to,
-       0, &DataParser::add_text, &DataParser::g3_to);
+       0, &DataParser::add_text, 0);
 
   init(s_g3_obs_dist_after_to, t_val,
-       s_g3_obs_dist_val, 0, s_g3_obs_dist_has_val,
-       0, &DataParser::add_text, &DataParser::g3_val);
+       s_g3_obs_dist_val, 0, s_g3_obs_dist_opt,
+       0, &DataParser::add_text, 0);
 
-  init(s_g3_obs_dist_has_val, t_stdev,
-       s_g3_obs_dist_stdev, 0, s_g3_obs_dist_has_variance,
-       0, &DataParser::add_text, &DataParser::g3_stdev);
+  init(s_g3_obs_dist_opt, t_stdev,
+       s_g3_obs_dist_opt_stdev, 0, 0,
+       0, &DataParser::optional_stdev, 0);
+ 
+  init(s_g3_obs_dist_opt, t_variance,
+       s_g3_obs_dist_opt_variance, 0, 0,
+       0, &DataParser::optional_variance, 0);
+ 
+  init(s_g3_obs_dist_opt, t_from_dh,
+       s_g3_obs_dist_opt_from_dh, 0, 0,
+       0, &DataParser::optional_from_dh, 0);
+ 
+  init(s_g3_obs_dist_opt, t_to_dh,
+       s_g3_obs_dist_opt_to_dh, 0, 0,
+       0, &DataParser::optional_to_dh, 0);
 
-  init(s_g3_obs_dist_has_val, t_variance,
-       s_g3_obs_dist_variance, 0, s_g3_obs_dist_has_variance,
-       0, &DataParser::add_text, &DataParser::g3_variance);
+  // .....  <g3-model> <obs> <vector>  ...............................
+
+  init(s_g3_obs, t_vector,
+       s_g3_obs_vector, s_g3_obs_vector_opt, 0,
+       0, 0, &DataParser::g3_obs_vector);
+  
+   init(s_g3_obs_vector, t_from,
+        s_g3_obs_vector_from, 0, s_g3_obs_vector_after_from,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_vector_after_from, t_to,
+        s_g3_obs_vector_to, 0, s_g3_obs_vector_after_to,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_vector_after_to, t_dx,
+        s_g3_obs_vector_dx, 0, s_g3_obs_vector_after_dx,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_vector_after_dx, t_dy,
+        s_g3_obs_vector_dy, 0, s_g3_obs_vector_after_dy,
+        0, &DataParser::add_text, 0);
+  
+   init(s_g3_obs_vector_after_dy, t_dz,
+        s_g3_obs_vector_dz, 0, s_g3_obs_vector_opt,
+        0, &DataParser::add_text, 0);
+  
+   init(s_g3_obs_vector_opt, t_from_dh,
+        s_g3_obs_vector_opt_from_dh, 0, 0,
+        0, &DataParser::optional_from_dh, 0);
+  
+   init(s_g3_obs_vector_opt, t_to_dh,
+        s_g3_obs_vector_opt_to_dh, 0, 0,
+        0, &DataParser::optional_to_dh, 0);
+
+  // .....  <g3-model> <obs> <xyz>  ..................................
+
+   init(s_g3_obs, t_xyz,
+        s_g3_obs_xyz, s_g3_obs_xyz_after_z, 0,
+        0, 0, &DataParser::g3_obs_xyz);
+
+   init(s_g3_obs_xyz, t_id,
+        s_g3_obs_xyz_id, 0, s_g3_obs_xyz_after_id,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_xyz_after_id, t_x,
+        s_g3_obs_xyz_x, 0, s_g3_obs_xyz_after_x,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_xyz_after_x, t_y,
+        s_g3_obs_xyz_y, 0, s_g3_obs_xyz_after_y,
+        0, &DataParser::add_text, 0);
+
+   init(s_g3_obs_xyz_after_y, t_z,
+        s_g3_obs_xyz_z, 0, s_g3_obs_xyz_after_z,
+        0, &DataParser::add_text, 0);
 
   // .....  <text>  ..................................................
  
-  init(  s_gama_data, t_text,
-         //--------------------
-         s_text, 0, 0,
-         0, &DataParser::add_text, &DataParser::text);
+  init(s_gama_data, t_text,
+       s_text, 0, 0,
+       0, &DataParser::add_text, &DataParser::text);
  
   // .....  <adj-input-data>  ........................................
 
-  init(  s_gama_data, t_adj_input_data, 
-         //------------------------------
-         s_adj_input_data_1, s_adj_input_data_5, 0,
-         &DataParser::adj_input_data, 0, &DataParser::adj_input_data,
-         s_adj_input_data_4);
+  init(s_gama_data, t_adj_input_data, 
+       s_adj_input_data_1, s_adj_input_data_5, 0,
+       &DataParser::adj_input_data, 0, &DataParser::adj_input_data,
+       s_adj_input_data_4);
 
   // .....  <sparse-mat>  ............................................
 
-  init(  s_adj_input_data_1, t_sparse_mat,
-       //-------------------------------
+  init(s_adj_input_data_1, t_sparse_mat,
        s_sparse_mat_1, s_sparse_mat_4, s_adj_input_data_2,
        0, 0, &DataParser::sparse_mat);
 
-  init(  s_sparse_mat_1, t_rows, 
-       //-----------------------
+  init(s_sparse_mat_1, t_rows, 
        s_sparse_mat_rows, 0, s_sparse_mat_2,
-       0, &DataParser::add_text, &DataParser::append_sp);
+       0, &DataParser::add_text, 0);
 
-  init(  s_sparse_mat_2, t_cols,
-       //-----------------------
+  init(s_sparse_mat_2, t_cols,
        s_sparse_mat_cols, 0, s_sparse_mat_3,
-       0, &DataParser::add_text, &DataParser::append_sp);
+       0, &DataParser::add_text, 0);
 
-  init(  s_sparse_mat_3, t_nonz,
-       //-----------------------
+  init(s_sparse_mat_3, t_nonz,
        s_sparse_mat_nonz, 0, s_sparse_mat_4,
        0, &DataParser::add_text, &DataParser::sparse_mat_nonz);
  
-  init(  s_sparse_mat_4, t_row, 
-       //----------------------
+  init(s_sparse_mat_4, t_row, 
        s_sparse_mat_row_1, s_sparse_mat_row_2, 0,
        &DataParser::sparse_mat_row, 0, &DataParser::sparse_mat_row);
 
-  init(  s_sparse_mat_row_1, t_nonz,
-       //--------------------------- 
+  init(s_sparse_mat_row_1, t_nonz,
        s_sparse_mat_row_nonz, 0, s_sparse_mat_row_2,
        0, &DataParser::add_text, &DataParser::sparse_mat_row_n);
 
-  init(  s_sparse_mat_row_2, t_int, 
-       //--------------------------
+  init(s_sparse_mat_row_2, t_int, 
        s_sparse_mat_row_int, 0, s_sparse_mat_row_3,
-       0, &DataParser::add_text, &DataParser::append_sp);
+       0, &DataParser::add_text, 0);
 
-  init(  s_sparse_mat_row_3, t_flt,
-       //--------------------------
+  init(s_sparse_mat_row_3, t_flt,
        s_sparse_mat_row_flt, 0, s_sparse_mat_row_2,
        0, &DataParser::add_text, &DataParser::sparse_mat_row_f);
 
   // ......  <block-diagonal>  .......................................
 
-  init(  s_adj_input_data_2, t_block_diagonal, 
-       //-----------------------------------
+  init(s_adj_input_data_2, t_block_diagonal, 
        s_block_diagonal_1, s_block_diagonal_3, s_adj_input_data_3,
        0, 0, &DataParser::block_diagonal);
 
-  init(  s_block_diagonal_1, t_blocks,
-       //-----------------------------
+  init(s_block_diagonal_1, t_blocks,
        s_block_diagonal_blocks, 0, s_block_diagonal_2,
-       0, &DataParser::add_text, &DataParser::append_sp);
+       0, &DataParser::add_text, 0);
 
-  init(  s_block_diagonal_2, t_nonz,
-       //---------------------------
+  init(s_block_diagonal_2, t_nonz,
        s_block_diagonal_nonz, 0, s_block_diagonal_3,
        0, &DataParser::add_text, &DataParser::block_diagonal_nonz);
 
-  init(  s_block_diagonal_3, t_block,
-       //----------------------------
+  init(s_block_diagonal_3, t_block,
        s_block_diagonal_block_1, s_block_diagonal_block_3, 0,
        0, 0, &DataParser::block_diagonal_block);
 
-  init(  s_block_diagonal_block_1, t_dim,
-       //--------------------------------
+  init(s_block_diagonal_block_1, t_dim,
        s_block_diagonal_block_d, 0, s_block_diagonal_block_2,
-       0, &DataParser::add_text, &DataParser::append_sp);
+       0, &DataParser::add_text, 0);
 
-  init(  s_block_diagonal_block_2, t_width,
-       //----------------------------------
+  init(s_block_diagonal_block_2, t_width,
        s_block_diagonal_block_w, 0, s_block_diagonal_block_3,
        0, &DataParser::add_text, &DataParser::block_diagonal_block_w);
 
-  init(  s_block_diagonal_block_3, t_flt,  
-       //--------------------------------
+  init(s_block_diagonal_block_3, t_flt,  
        s_block_diagonal_block_f, 0, 0,
        0, &DataParser::add_text, &DataParser::block_diagonal_vec_flt);
 
   // ......  <vector>  ...............................................
 
-  init(  s_adj_input_data_3, t_vector, 
-       //--------------------------
+  init(s_adj_input_data_3, t_vector, 
        s_vector_1, s_vector_2, s_adj_input_data_4,
        0, 0, &DataParser::vector);
 
-  init(  s_vector_1, t_dim, 
-       //------------------
+  init(s_vector_1, t_dim, 
        s_vector_dim, 0, s_vector_2,
        0, &DataParser::add_text, &DataParser::vector_dim);
 
-  init(  s_vector_2, t_flt, 
-       //------------------
+  init(s_vector_2, t_flt, 
        s_vector_flt, 0, 0,
        0, &DataParser::add_text, &DataParser::vector_flt);
 
   // ......  <array>  ................................................
 
-  init(  s_adj_input_data_4, t_array,
-       //--------------------------
+  init(s_adj_input_data_4, t_array,
        s_array_1, s_array_2, s_adj_input_data_5,
        0, 0, &DataParser::array);
 
-  init(  s_array_1, t_dim,
-       //-----------------
+  init(s_array_1, t_dim,
        s_array_dim, 0, s_array_2,
        0, &DataParser::add_text, &DataParser::array_dim);
 
-  init(  s_array_2, t_int,
-       //-----------------
+  init(s_array_2, t_int,
        s_array_int, 0, 0,
        0, &DataParser::add_text, &DataParser::array_int);
          
-
   // .................................................................
 }
 
@@ -584,6 +463,7 @@ DataParser::data_tag DataParser::tag(const char* c)
       if (!strcmp(c, "array"          )) return t_array;
       break;
     case 'b':
+      if (!strcmp(c, "b"              )) return t_b;
       if (!strcmp(c, "band"           )) return t_band;
       if (!strcmp(c, "block-diagonal" )) return t_block_diagonal;
       if (!strcmp(c, "blocks"         )) return t_blocks;
@@ -592,15 +472,7 @@ DataParser::data_tag DataParser::tag(const char* c)
     case 'c' :
       if (!strcmp(c, "cols"           )) return t_cols;
       if (!strcmp(c, "constr"         )) return t_constr;
-      if (!strcmp(c, "constr-position")) return t_constr_p;
-      if (!strcmp(c, "constr-height"  )) return t_constr_h;
       if (!strcmp(c, "cov-mat"        )) return t_covmat;
-      if (!strcmp(c, "cxx"            )) return t_cxx;
-      if (!strcmp(c, "cxy"            )) return t_cxy;
-      if (!strcmp(c, "cxz"            )) return t_cxz;
-      if (!strcmp(c, "cyy"            )) return t_cyy;
-      if (!strcmp(c, "cyz"            )) return t_cyz;
-      if (!strcmp(c, "czz"            )) return t_czz;
       break;
     case 'd' :
       if (!strcmp(c, "dim"            )) return t_dim;
@@ -609,28 +481,33 @@ DataParser::data_tag DataParser::tag(const char* c)
       if (!strcmp(c, "dy"             )) return t_dy;
       if (!strcmp(c, "dz"             )) return t_dz;
       break;
+    case 'e':
+      if (!strcmp(c, "e"              )) return t_e;
+      break;
     case 'f' :
       if (!strcmp(c, "fixed"          )) return t_fixed;
-      if (!strcmp(c, "fixed-position" )) return t_fixed_p;
-      if (!strcmp(c, "fixed-height"   )) return t_fixed_h;
       if (!strcmp(c, "flt"            )) return t_flt;
       if (!strcmp(c, "from"           )) return t_from;
+      if (!strcmp(c, "from-dh"        )) return t_from_dh;
       if (!strcmp(c, "free"           )) return t_free;
-      if (!strcmp(c, "free-position"  )) return t_free_p;
-      if (!strcmp(c, "free-height"    )) return t_free_h;
       break;
     case 'g' :
       if (!strcmp(c, "g3-model"       )) return t_g3_model;
       if (!strcmp(c, "gnu-gama-data"  )) return t_gama_data;
       break;
     case 'h':
+      if (!strcmp(c, "h"              )) return t_h;
       if (!strcmp(c, "height"         )) return t_height;
       break;
     case 'i':
       if (!strcmp(c, "id"             )) return t_id;
       if (!strcmp(c, "int"            )) return t_int;
       break;
+    case 'l':
+      if (!strcmp(c, "l"              )) return t_l;
+      break;
     case 'n':
+      if (!strcmp(c, "n"              )) return t_n;
       if (!strcmp(c, "nonz"           )) return t_nonz;
       break;
     case 'o':
@@ -650,8 +527,10 @@ DataParser::data_tag DataParser::tag(const char* c)
     case 't' :
       if (!strcmp(c, "text"           )) return t_text;
       if (!strcmp(c, "to"             )) return t_to;
+      if (!strcmp(c, "to-dh"          )) return t_to_dh;
       break;
     case 'u' :
+      if (!strcmp(c, "u"              )) return t_u;
       if (!strcmp(c, "unused"         )) return t_unused;
       break;
     case 'v' :
@@ -664,6 +543,7 @@ DataParser::data_tag DataParser::tag(const char* c)
       break;
     case 'x' :
       if (!strcmp(c, "x"              )) return t_x;
+      if (!strcmp(c, "xyz"            )) return t_xyz;
       break;
     case 'y' :
       if (!strcmp(c, "y"              )) return t_y;
@@ -722,32 +602,11 @@ int DataParser::white_spaces(const char* s, int len)
 
 int DataParser::add_text(const char* s, int len)
 {
+  text_buffer += ' ';
   text_buffer += string(s, len);
   return 0;
 }
 
-int DataParser::append_sp(const char *name)
-{
-  text_buffer += ' ';
-  return end_tag(name);
-}
-
-int DataParser::g3_from(const char *name)
-{
-  g3from = text_buffer;
-  text_buffer.erase();
-
-  return  end_tag(name);
-}
-
-int DataParser::g3_to(const char *name)
-{
-  g3to = text_buffer;
-  text_buffer.erase();
-
-  return  end_tag(name);
-}
-
 int DataParser::g3_get_float(const char *name, double& d)
 {
   istringstream inp(text_buffer.c_str());
@@ -762,24 +621,6 @@ int DataParser::g3_get_float(const char *name, double& d)
                string(name) + string(">"));
 }
 
-int DataParser::g3_val(const char *name) 
-{ 
-  return g3_get_float(name, g3val); 
-}
-
-int DataParser::g3_stdev(const char *name) 
-{ 
-  double tmp;
-  int result = g3_get_float(name, tmp); 
-  g3variance = tmp*tmp;
-  return result;
-}
-
-int DataParser::g3_variance(const char *name) 
-{ 
-  return g3_get_float(name, g3variance);
-}
-
 bool DataParser::pure_data(std::istream& istr)
 {
   if (istr.eof()) return true;
@@ -1085,53 +926,68 @@ int DataParser::g3_model(const char *name)
   return  end_tag(name);
 }
 
-int DataParser::g3_vector(const char *name, const char **atts)
+int DataParser::g3_param_unused(const char *name, const char **atts)
 {
   no_attributes( name, atts );
   state = next[state][tag(name)];
 
-  g3from = "";
-  g3to   = "";
+  local_state.set_unused();
 
-  return 0;
+  return  0;
 }
 
-int DataParser::g3_vector(const char *name)
+int DataParser::g3_param_fixed(const char *name, const char **atts)
 {
-  stringstream istr(text_buffer);
-  double dx, dy, dz, cxx, cxy, cxz, cyy, cyz, czz;
+  no_attributes( name, atts );
+  state = next[state][tag(name)];
 
-  if (!(istr >> dx >> dy >> dz >> cxx >> cxy >> cxz >> cyy >> cyz >> czz))
-    {
-      return error("### bad format of numerical data in <vector>");
-    }
+  local_state.set_fixed();
 
-  text_buffer.erase();
+  return  0;
+}
 
-  using namespace g3;
-  
-  Vector* v = new Vector(dx, dy, dz);
-  v->from = g3from;
-  v->to   = g3to;
+int DataParser::g3_param_free(const char *name, const char **atts)
+{
+  no_attributes( name, atts );
+  state = next[state][tag(name)];
 
-  // g3::Model::ObservationData *obs = g3model->obs;
-  // Vectors* vectors = new Vectors(obs);
-  // 
-  // vectors->add(v);
-  // vectors->covariance_matrix.reset(3,2);
-  // vectors->covariance_matrix(1,1) = cxx;
-  // vectors->covariance_matrix(1,2) = cxy;
-  // vectors->covariance_matrix(1,3) = cxz;
-  // vectors->covariance_matrix(2,2) = cyy;
-  // vectors->covariance_matrix(2,3) = cyz;
-  // vectors->covariance_matrix(3,3) = czz;
-  // 
-  // obs->CL.push_back(vectors);
+  local_state.set_free();
+
+  return  0;
+}
+
+int DataParser::g3_param_constr(const char *name, const char **atts)
+{
+  no_attributes( name, atts );
+  state = next[state][tag(name)];
+
+  local_state.set_constr();
+
+  return  0;
+}
+
+int DataParser::g3_param_n(const char *name)
+{
+  global_state_N.set_state(local_state);
 
   return  end_tag(name);
 }
 
-int DataParser::g3_point_id(const char *name)
+int DataParser::g3_param_e(const char *name)
+{
+  global_state_E.set_state(local_state);
+
+  return  end_tag(name);
+}
+
+int DataParser::g3_param_u(const char *name)
+{
+  global_state_U.set_state(local_state);
+
+  return  end_tag(name);
+}
+
+std::string DataParser::g3_get_id(std::string err)
 {
   char    c;
   string id;
@@ -1142,82 +998,90 @@ int DataParser::g3_point_id(const char *name)
   while (i!=e && !isspace((c = *i))) { ++i; id += c; }
   while (i!=e &&  isspace((c = *i))) { ++i;          }
 
-  if (i!=e || id.empty())
-    return error("### bad point name in <id> tag");
+  if (i!=e || id.empty()) error(err);
 
   text_buffer.erase();
+  return id;
+}
+
+int DataParser::g3_point(const char *name)
+{
+  // checks consistency of the current point
+
+  if (!point->N.cmp_state(point->E)) 
+    return error("### parameters N and E do not have common status");
+
+  return  end_tag(name);
+}
+
+int DataParser::g3_point_id(const char *name)
+{
+  string id = g3_get_id("### bad point name in <id> tag");
 
   point = g3model->get_point(id);
 
-  return  end_tag(name);
-}
-
-int DataParser::g3_point_unused(const char *name)
-{
-  point->set_unused();
+  point->N.set_state(global_state_N);
+  point->E.set_state(global_state_E);
+  point->U.set_state(global_state_U);
 
   return  end_tag(name);
 }
 
-int DataParser::g3_point_fixed(const char *name)
+int DataParser::g3_point_param_n(const char *name)
 {
-  point->set_fixed_position();
+  point->N.set_state(local_state);
 
   return  end_tag(name);
 }
 
-int DataParser::g3_point_fixed_p(const char *name)
+int DataParser::g3_point_param_e(const char *name)
 {
-  point->set_fixed_horizontal_position();
+  point->E.set_state(local_state);
 
   return  end_tag(name);
 }
 
-int DataParser::g3_point_fixed_h(const char *name)
+int DataParser::g3_point_param_u(const char *name)
 {
-  point->set_fixed_height();
+  point->U.set_state(local_state);
 
   return  end_tag(name);
 }
 
-int DataParser::g3_point_free(const char *name)
+int DataParser::g3_point_b(const char *name)
 {
-  point->set_free_position();
+  if (!deg2gon(text_buffer, blh.b))
+    {
+      return error("### bad format of numerical data in <point> <b> ");
+    }
+  text_buffer.erase();
 
   return  end_tag(name);
 }
 
-int DataParser::g3_point_free_p(const char *name)
+int DataParser::g3_point_l(const char *name)
 {
-  point->set_free_horizontal_position();
+  if (!deg2gon(text_buffer, blh.l))
+    {
+      return error("### bad format of numerical data in <point> <l> ");
+    }
+  text_buffer.erase();
 
   return  end_tag(name);
 }
 
-int DataParser::g3_point_free_h(const char *name)
+int DataParser::g3_point_h(const char *name)
 {
-  point->set_free_height();
+  stringstream istr(text_buffer);
+  if (!(istr >> blh.h))
+    {
+      return error("### bad format of numerical data in <point> <h> ");
+    }
+  text_buffer.erase();
 
-  return  end_tag(name);
-}
-
-int DataParser::g3_point_constr(const char *name)
-{
-  point->set_constr_position();
-
-  return  end_tag(name);
-}
-
-int DataParser::g3_point_constr_p(const char *name)
-{
-  point->set_constr_horizontal_position();
-
-  return  end_tag(name);
-}
-
-int DataParser::g3_point_constr_h(const char *name)
-{
-  point->set_constr_height();
+  blh.b *= GON_TO_RAD;
+  blh.l *= GON_TO_RAD;
+  point->set_blh(blh.b, blh.l, blh.h);
 
   return  end_tag(name);
 }
@@ -1269,31 +1133,51 @@ int DataParser::g3_obs(const char *name, const char **atts)
 int DataParser::g3_obs(const char *name)
 {
   using namespace g3;
-  int cov_dim  = g3obs_cluster->covariance_matrix.dim();
-  int obs_dim  = g3var_list.size();
 
-  if (cov_dim == 0)
+  int obs_dim = 0; 
+  for (int i=0, N=g3obs_cluster->observation_list.size(); i<N; i++)
     {
-      // implicit diagonal covariance matrix
-      
-      g3obs_cluster->covariance_matrix.reset(obs_dim, 0);
-      std::list<double>::const_iterator ivar = g3var_list.begin();
-      double var;
-      for (int i=1; i<=obs_dim; i++)
-        {
-          var = *ivar;
-          g3obs_cluster->covariance_matrix(i,i) = var;
-          ++ivar;
-        }
+      g3::Observation* obs = g3obs_cluster->observation_list[i];
+      obs_dim += obs->dimension();
     }
 
+  int cov_dim  = 0;
+  int cov_band = 0; 
+  typedef std::list<g3Cov>::const_iterator Iterator;
+  for (Iterator i=g3cov_list.begin(), e=g3cov_list.end(); i!=e; ++i)
+    {
+      const g3Cov& cov = *i;
+
+      cov_dim += cov.dim();
+      if (int(cov.bandWidth()) > cov_band) cov_band = cov.bandWidth();
+    }
+
+  if (obs_dim == 0)       return error("### no observations in <obs>");
+  if (obs_dim != cov_dim) return error("### bad covariance matrix dimension");
+
+  g3obs_cluster->covariance_matrix.reset(cov_dim, cov_band);
+  g3obs_cluster->covariance_matrix.set_zero();
+
+  int offset = 0;
+  for (Iterator i=g3cov_list.begin(), e=g3cov_list.end(); i!=e; ++i)
+    {
+      const g3Cov& cov = *i;
+
+      for (int i=1; i<=cov.dim(); i++)
+        for (int j=0; j<=cov.bandWidth() && i+j<=cov.dim(); j++)
+          g3obs_cluster->covariance_matrix(offset+i, offset+i+j) = cov(i, i+j);
+
+      offset += cov.dim();
+    }
+
+  /* TODO: !!! here we should better test Cholesky decomposition !!! */
   for (int N=g3obs_cluster->covariance_matrix.dim(), i=1; i<=N; i++)
     if(g3obs_cluster->covariance_matrix(i,i) <= 0)
       return error("### zero or negative variance");
 
   g3obs_cluster->update();
   g3model->obsdata.CL.push_back(g3obs_cluster);
-  g3var_list.clear();
+  g3cov_list.clear();
 
   return  end_tag(name);
 }
@@ -1307,34 +1191,142 @@ int DataParser::g3_obs_cov(const char *name)
 
   if (!(istr >> d >> b))  return error("### bad cov-mat");
 
-  g3obs_cluster->covariance_matrix.reset(d, b);
+  g3Cov cov(d, b);
+  cov.set_zero();
   for (int i=1; i<=d; i++)          // upper triangular matrix by rows
     for (int j=i; j<=i+b && j<=d; j++)
       if (istr >> f)
-        g3obs_cluster->covariance_matrix(i,j) = f;
-      else
-        return error("### bad cov-mat / some data missing");
+        cov(i,j) = f;
+      else        return error("### bad cov-mat / some data missing");
 
   if (!pure_data(istr)) return error("### bad cov-mat / redundant data");
 
   text_buffer.clear();
+  g3cov_list.push_back(cov);
+
   return  end_tag(name);
 }
 
+int DataParser::optional_stdev(const char *s, int len)
+{
+  using namespace g3;
+  stringstream istr(string(s, len));
+  g3Cov   cov(1,0);
+  double  f;
+  
+  if (!pure_data(istr >> f)) return error("### bad <stdev>");
+
+  cov(1,1) = f*f;
+  g3cov_list.push_back(cov);
+
+  return 0;
+}
+
+int DataParser::optional_variance(const char *s, int len)
+{
+  using namespace g3;
+  stringstream istr(string(s, len));
+  g3Cov   cov(1,0);
+  double  f;
+  
+  if (!pure_data(istr >> f)) return error("### bad <variance>");
+
+  cov(1,1) = f;
+  g3cov_list.push_back(cov);
+
+  return 0;
+}
+
+int DataParser::optional_from_dh(const char *s, int len)
+{
+  using namespace g3;
+  stringstream istr(string(s,len));
+
+  if (pure_data(istr >> from_dh)) return 0;
+
+  return error("### bad data in <from-dh>");
+}
+
+int DataParser::optional_to_dh(const char *s, int len)
+{
+  using namespace g3;
+  stringstream istr(string(s,len));
+
+  if (pure_data(istr >> to_dh)) return 0;
+
+  return error("### bad data in <to-dh>");
+}
+
 int DataParser::g3_obs_dist(const char *name)
 {
   using namespace g3;  
-  g3var_list.push_back(g3variance);
-  g3variance = 0;
+  stringstream istr(text_buffer);
+  string       from, to;
+  double       val;
 
-  Distance* distance = new Distance;
-  distance->from = g3from;
-  distance->to   = g3to;
-  distance->set(g3val);
-  g3obs_cluster->observation_list.push_back(distance);  
+  if (pure_data(istr >> from >> to >> val))
+    {
+      text_buffer.clear();
 
-  return  end_tag(name);
+      Distance* distance = new Distance;
+      distance->from = from;
+      distance->to   = to;
+      distance->set(val);
+      distance->from_dh = optional(from_dh);
+      distance->to_dh   = optional(to_dh);
+      g3obs_cluster->observation_list.push_back(distance);  
+
+      return  end_tag(name);
+    }
+
+  return error("### bad <distance>");
 }
 
+int DataParser::g3_obs_vector(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  string from, to;
+  double dx, dy, dz;
 
-#endif
+  if (pure_data(istr >> from >> to >> dx >> dy >> dz))
+    {
+      text_buffer.clear();
+
+      Vector* vector = new Vector;
+      vector->from = from;
+      vector->to   = to;
+      vector->set_dxyz(dx, dy, dz);
+      vector->from_dh = optional(from_dh);
+      vector->to_dh   = optional(to_dh);
+
+      g3obs_cluster->observation_list.push_back(vector);
+
+      return end_tag(name);
+    }
+  
+  return error("### bad <vector>");
+}
+
+int DataParser::g3_obs_xyz(const char *name)
+{
+  using namespace g3;  
+  stringstream istr(text_buffer);
+  string id;
+  double x, y, z;
+
+  if (pure_data(istr >> id >> x >> y >> z))
+    {
+      text_buffer.clear();
+
+      XYZ* xyz = new XYZ;
+      xyz->id  = id;
+      xyz->set_xyz(x, y, z);
+
+      g3obs_cluster->observation_list.push_back(xyz);
+
+      return end_tag(name);
+    }
+  
+  return error("### bad <xyz>");
+}
