@@ -20,7 +20,7 @@
 */
 
 /*
- *  $Id: dataparser.h,v 1.15 2004/01/05 19:07:12 cepek Exp $
+ *  $Id: dataparser.h,v 1.19 2004/04/21 16:49:39 cepek Exp $
  */
 
 #ifndef GNU_Gama_GaMa_XML_DataParser__data_parser__dataparser___h_
@@ -71,49 +71,35 @@ namespace GNU_gama {
           s_gama_data,
           s_g3_model,
 
+          // ..................................................
+
+          s_g3_param,
+          s_g3_param_n,
+          s_g3_param_e,
+          s_g3_param_u,
+
+          // ..................................................
+
           s_g3_point_1,
           s_g3_point_id,
           s_g3_point_2,
+          s_g3_point_b,
+          s_g3_point_after_b,
+          s_g3_point_l,
+          s_g3_point_after_l,  
+          s_g3_point_h,
           s_g3_point_x,
           s_g3_point_after_x,
           s_g3_point_y,
           s_g3_point_after_y,
           s_g3_point_z,
           s_g3_point_height,
-          s_g3_point_unused,
-          s_g3_point_fixed,
-          s_g3_point_fixed_p,
-          s_g3_point_fixed_h,
-          s_g3_point_free,
-          s_g3_point_free_p,
-          s_g3_point_free_h,
-          s_g3_point_constr,
-          s_g3_point_constr_p,
-          s_g3_point_constr_h,
+          s_g3_point_param,
+          s_g3_point_param_n,
+          s_g3_point_param_e,
+          s_g3_point_param_u,
 
-          s_g3_vector,
-          s_g3_vector_from,
-          s_g3_vector_after_from,
-          s_g3_vector_to,
-          s_g3_vector_after_to,
-          s_g3_vector_dx,
-          s_g3_vector_after_dx,
-          s_g3_vector_dy,
-          s_g3_vector_after_dy,
-          s_g3_vector_dz,
-          s_g3_vector_after_dz,
-          s_g3_vector_cxx,
-          s_g3_vector_after_cxx,
-          s_g3_vector_cxy,
-          s_g3_vector_after_cxy,
-          s_g3_vector_cxz,
-          s_g3_vector_after_cxz,
-          s_g3_vector_cyy,
-          s_g3_vector_after_cyy,
-          s_g3_vector_cyz,
-          s_g3_vector_after_cyz,
-          s_g3_vector_czz,
-          s_g3_vector_after_czz,
+          // ..................................................
 
           s_g3_obs,
           s_g3_obs_covmat,
@@ -122,25 +108,55 @@ namespace GNU_gama {
           s_g3_obs_covmat_band,
           s_g3_obs_covmat_after_band,
           s_g3_obs_covmat_flt,
-          s_g3_obs_after_covmat,
+
           s_g3_obs_dist,
           s_g3_obs_dist_from,
           s_g3_obs_dist_after_from,
           s_g3_obs_dist_to,
           s_g3_obs_dist_after_to,
           s_g3_obs_dist_val,
-          s_g3_obs_dist_has_val,
-          s_g3_obs_dist_stdev,
-          s_g3_obs_dist_variance,
-          s_g3_obs_dist_has_variance,
+          s_g3_obs_dist_opt,
+          s_g3_obs_dist_opt_stdev,
+          s_g3_obs_dist_opt_variance,
+          s_g3_obs_dist_opt_from_dh,
+          s_g3_obs_dist_opt_to_dh,
+
+          s_g3_obs_vector,
+          s_g3_obs_vector_from,
+          s_g3_obs_vector_after_from,
+          s_g3_obs_vector_to,
+          s_g3_obs_vector_after_to,
+          s_g3_obs_vector_dx,
+          s_g3_obs_vector_after_dx,
+          s_g3_obs_vector_dy,
+          s_g3_obs_vector_after_dy,
+          s_g3_obs_vector_dz,
+          s_g3_obs_vector_opt,
+          s_g3_obs_vector_opt_from_dh,
+          s_g3_obs_vector_opt_to_dh,
+
+          s_g3_obs_xyz,
+          s_g3_obs_xyz_id,
+          s_g3_obs_xyz_after_id,
+          s_g3_obs_xyz_x,
+          s_g3_obs_xyz_after_x,
+          s_g3_obs_xyz_y,
+          s_g3_obs_xyz_after_y,
+          s_g3_obs_xyz_z,
+          s_g3_obs_xyz_after_z,
+
+          // ..................................................
 
           s_text,
 
+          // ..................................................
+
           s_adj_input_data_1,
           s_adj_input_data_2,
           s_adj_input_data_3,
           s_adj_input_data_4,
           s_adj_input_data_5,
+
           s_sparse_mat_1,
           s_sparse_mat_rows,
           s_sparse_mat_2,
@@ -154,6 +170,7 @@ namespace GNU_gama {
           s_sparse_mat_row_int,
           s_sparse_mat_row_3,
           s_sparse_mat_row_flt,
+
           s_block_diagonal_1,
           s_block_diagonal_blocks,
           s_block_diagonal_2,
@@ -165,10 +182,12 @@ namespace GNU_gama {
           s_block_diagonal_block_w,
           s_block_diagonal_block_3,
           s_block_diagonal_block_f,
+
           s_vector_1,
           s_vector_dim,
           s_vector_2,
           s_vector_flt,
+
           s_array_1,
           s_array_dim,
           s_array_2,
@@ -180,39 +199,33 @@ namespace GNU_gama {
         {
           t_adj_input_data,
           t_array,
+          t_b,
           t_band,
           t_block,
           t_block_diagonal,
           t_blocks,
           t_cols,
           t_constr,
-          t_constr_p,
-          t_constr_h,
           t_covmat,
-          t_cxx,
-          t_cxy,
-          t_cxz,
-          t_cyy,
-          t_cyz,
-          t_czz,
           t_dim,
           t_dist,
           t_dx,
           t_dy,
           t_dz,
+          t_e,
           t_flt,
           t_fixed,
-          t_fixed_p,
-          t_fixed_h,
           t_free,
-          t_free_p,
-          t_free_h,
           t_from,
+          t_from_dh,
           t_g3_model,
           t_gama_data,
+          t_h,
           t_height,
           t_id,
           t_int,
+          t_l,
+          t_n,
           t_nonz,
           t_obs,
           t_point,
@@ -222,12 +235,15 @@ namespace GNU_gama {
           t_stdev,
           t_text,
           t_to,
+          t_to_dh,
+          t_u,
           t_unknown,
           t_val,
           t_variance,
           t_vector,
           t_width,
           t_x,
+          t_xyz,
           t_y,
           t_z,
           t_unused
@@ -249,26 +265,36 @@ namespace GNU_gama {
       int gama_data             (const char *name, const char **atts);
       int g3_model              (const char *name, const char **atts);
       int g3_model              (const char *name);
+
+      int g3_param_unused       (const char *name, const char **atts);
+      int g3_param_fixed        (const char *name, const char **atts);
+      int g3_param_free         (const char *name, const char **atts);
+      int g3_param_constr       (const char *name, const char **atts);
+
+      int g3_param_n            (const char *name);
+      int g3_param_e            (const char *name);
+      int g3_param_u            (const char *name);
+
+      int g3_point              (const char *name);
       int g3_point_id           (const char *name);
+      int g3_point_b            (const char *name);
+      int g3_point_l            (const char *name);
+      int g3_point_h            (const char *name);
       int g3_point_z            (const char *name);
       int g3_point_height       (const char *name);
-      int g3_point_unused       (const char *name);
-      int g3_point_fixed        (const char *name);
-      int g3_point_fixed_p      (const char *name);
-      int g3_point_fixed_h      (const char *name);
-      int g3_point_free         (const char *name);
-      int g3_point_free_p       (const char *name);
-      int g3_point_free_h       (const char *name);
-      int g3_point_constr       (const char *name);
-      int g3_point_constr_p     (const char *name);
-      int g3_point_constr_h     (const char *name);
-      int g3_vector             (const char *name, const char **atts);
-      int g3_vector             (const char *name);
+      int g3_point_param_n      (const char *name);
+      int g3_point_param_e      (const char *name);
+      int g3_point_param_u      (const char *name);
+
       int g3_obs                (const char *name, const char **atts);
       int g3_obs                (const char *name);
       int g3_obs_cov            (const char *name);
       int g3_obs_dist           (const char *name);
+      int g3_obs_vector         (const char *name);
+      int g3_obs_xyz            (const char *name);
+
       int text                  (const char *name);
+
       int adj_input_data        (const char *name, const char **atts);
       int adj_input_data        (const char *name);
       int sparse_mat            (const char *name);
@@ -295,12 +321,11 @@ namespace GNU_gama {
       int parser_error (const char *name, const char **atts);
       int start_tag    (const char *name, const char **atts);
       int white_spaces (const char *name, int len);
-      int append_sp    (const char *name);
-      int g3_from      (const char *name);
-      int g3_to        (const char *name);
-      int g3_val       (const char *name);
-      int g3_stdev     (const char *name);
-      int g3_variance  (const char *name);
+
+      int optional_stdev   (const char *name, int len);
+      int optional_variance(const char *name, int len);
+      int optional_from_dh (const char *name, int len);
+      int optional_to_dh   (const char *name, int len);
 
       void init(int state, int tag, 
                 int next_state, int end_state, int after_state,
@@ -310,21 +335,32 @@ namespace GNU_gama {
       bool pure_data(std::istream&);   // test for trailing junk in input data
 
 
-      // DataObject::g3_model
+      // ***  DataObject::g3_model ***
 
       g3::Model*         g3model;
       g3::Point::Name    g3from;
       g3::Point::Name    g3to;
       g3::ObsCluster*    g3obs_cluster;
       double             g3val;
-      double             g3variance;
-      std::list<double>  g3var_list;
 
-      // DataObject::Text
+      double from_dh, to_dh;
+      double optional(double& attr)
+      {
+        double tmp = attr;  attr=0; return tmp;
+      }
+
+      typedef g3::Model::ObservationType::CovarianceMatrix g3Cov;
+      std::list<g3Cov>   g3cov_list;
+
+      std::string g3_get_id(std::string err);
+
+
+      // ***  DataObject::Text  ***
 
       std::string      text_buffer;
 
-      // DataObject::AdjInput
+
+      // ***  DataObject::AdjInput  ***
 
       GNU_gama::SparseMatrix <> *adj_sparse_mat;
       GNU_gama::BlockDiagonal<> *adj_block_diagonal;
@@ -345,6 +381,15 @@ namespace GNU_gama {
       std::size_t      bd_vector_dim;
       g3::Point        *point;
 
+      g3::Parameter    local_state;
+      g3::Parameter    global_state_N;
+      g3::Parameter    global_state_E;
+      g3::Parameter    global_state_U;
+
+      struct {
+        double b, l, h;
+      } blh;
+
     };
 }
 
